#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#define RQ_SET_LED         0
#define RQ_SET_LED_VALUE   1
#define RQ_GET_SWITCH      2
#define RQ_GET_RTHUMB      3
#define RQ_GET_RINDEX      4
#define RQ_GET_RMID        5
#define RQ_GET_LTHUMB      6
#define RQ_GET_LINDEX      7
#define RQ_GET_LMID        8

/*USB interface*/

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
    static uint8_t switch_state;
    static uint16_t Rthumb;
    static uint16_t Rindex;
    static uint16_t Rmid;
    static uint16_t Lthumb;
    static uint16_t Lindex;
    static uint16_t Lmid;
    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_state = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];
        set_led(led_no, led_state);
        return 0;
    }

    else if (rq->bRequest == RQ_GET_SWITCH)
    {
        switch_state = SWITCH_PRESSED();

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = &switch_state;

        /* return the number of bytes of data to be returned to host */
        return 1;
    }
    else if (rq->bRequest == RQ_SET_LED_VALUE)
    {
        uint8_t led_value  = rq->wValue.bytes[0];
        set_led_value(led_value);
        return 0;
    }
    else if (rq->bRequest == RQ_GET_RTHUMB)
    {
        Rthumb  = read_adc(PC0);
        usbMsgPtr = &Rthumb;
        return 2;
    }
    else if (rq->bRequest == RQ_GET_RINDEX)
    {
        Rindex  = read_adc(PC1);
        usbMsgPtr = &Rindex;
        return 2;
    }
    else if (rq->bRequest == RQ_GET_RMID)
    {
        Rmid  = read_adc(PC2);
        usbMsgPtr = &Rmid;
        return 2;
    }
   else if (rq->bRequest == RQ_GET_LTHUMB)
    {
        Lthumb  = read_adc(PC3);
        usbMsgPtr = &Lthumb;
        return 2;
    }
   else if (rq->bRequest == RQ_GET_LINDEX)
    {
        Lindex  = read_adc(PC4);
        usbMsgPtr = &Lindex;
        return 2;
    }
    else if (rq->bRequest == RQ_GET_LMID)
    {
        Lmid  = read_adc(PC5);
        usbMsgPtr = &Lmid;
        return 2;
    }
    /* default for not implemented requests: return no data back to host */
    return 0;
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peri();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */



